import argparse


def parse():
    """Parses arguments."""
    parser = argparse.ArgumentParser(
        description="Compares two configuration files and shows a difference."
    )
    parser.add_argument("first_file")
    parser.add_argument("second_file")
    parser.add_argument("-f", "--format", default='stylish',
                        type=str, help="output format (default: 'stylish')")
    args = parser.parse_args()
    return (args.first_file, args.second_file, args.format)



from .plain import make_plain
from .stylish import make_stylish
from .json import make_json


def get_formatter(name_format):
    """Returns the formatter"""
    if name_format == 'plain':
        return make_plain
    elif name_format == 'json':
        return make_json
    return make_stylish



def make_plain(data): # noqa:901
    """Formatter function. Shows flat format.
    Returns a string.
    """

    def iter_(current_value, path):
        lines = []
        for i in current_value:
            if path and not path.endswith('.'):
                path += '.'
            if i.get('type') == 'nested':
                lines.append(iter_(i['value'], path=path + i['key']))
            elif i.get('type') == 'added':
                lines.append(f"Property '{path + i['key']}' was added with value: {to_str(i['value'])}") # noqa:501
            elif i.get('type') == 'removed':
                lines.append(f"Property '{path + i['key']}' was removed")
            elif i.get('type') == 'updated':
                lines.append(f"Property '{path + i['key']}' was updated. From {to_str(i['value1'])} to {to_str(i['value2'])}") # noqa:501

        return '\n'.join(lines)

    return iter_(data, '')


def to_str(value):
    """Converts the value to the desired format."""
    if isinstance(value, dict):
        return "[complex value]"
    elif value is True or value is False:
        return str(value).lower()
    elif value is None:
        return 'null'
    elif isinstance(value, int):
        return value
    return f"'{str(value)}'"



import pytest

import os

from gendiff.generate_diff import generate_diff


def get_fixture_path(file_name):
    """Returns the full path of the fixture."""
    current_dir = os.path.dirname(os.path.abspath(__file__))
    return os.path.join(current_dir, 'fixtures', file_name)


def read(file_path):
    """Reads the file."""
    with open(file_path, 'r') as f:
        result = f.read()
    return result


@pytest.mark.parametrize("file_name1, file_name2, formatter, expected_result", [
    ["file1.json", "file2.json", "stylish", "plain.txt"],
    ["file1.yaml", "file2.yml", "stylish", "plain.txt"],
    ["file1.json", "file2.yml", "stylish", "plain.txt"],
    ["file1_nested.json", "file2_nested.json", "stylish", "nested.txt"],
    ["file1_nested.yml", "file2_nested.yaml", "stylish", "nested.txt"],
    ["file1_nested.yml", "file2_nested.json", "stylish", "nested.txt"],
    ["file1_nested.json", "file2_nested.json", "plain", "flat_format.txt"],
    ["file1_nested.yml", "file2_nested.yaml", "plain", "flat_format.txt"],
    ["file1_nested.yml", "file2_nested.json", "plain", "flat_format.txt"],
    ["file1_nested.yml", "file2_nested.json", "json", "file_json.txt"],
    ["file1_nested.json", "file2_nested.json", "json", "file_json.txt"],
    ["file1_nested.yml", "file2_nested.yaml", "json", "file_json.txt"],
])
def test_generate_diff(file_name1, file_name2, formatter, expected_result):
    """Tests the operation of the function generate_diff.
    Accepts different file paths and formatters"""
    with open(get_fixture_path(expected_result)) as file:
        assert generate_diff(
            get_fixture_path(file_name1),
            get_fixture_path(file_name2),
            formatter
        ) == file.read()


def test_wrong_format():
    """Tests for unsupported format error."""
    with pytest.raises(ValueError) as e:
        generate_diff(
            get_fixture_path("file1_nested.json"),
            get_fixture_path("file.txt"),
        )

    assert str(e.value) == 'unsupported file format'



def get_different(data1, data2):
    """Returns a list with a difference."""
    list_diff = []
    keys_data1 = {key for key in data1.keys()}
    keys_data2 = {key for key in data2.keys()}
    set_keys = sorted(keys_data1 | keys_data2)
    for key in set_keys:
        if key not in data2:
            list_diff.append({
                "key": key,
                "type": 'removed',
                "value": data1[key],
            })
            continue
        elif key not in data1:
            list_diff.append({
                "key": key,
                "type": 'added',
                "value": data2[key],
            })
            continue
        elif isinstance(data1[key], dict) and isinstance(data2[key], dict):
            list_diff.append({
                "key": key,
                "type": 'nested',
                "value": get_different(data1[key], data2[key])
            })
            continue
        elif data1[key] == data2[key]:
            list_diff.append({
                "key": key,
                "type": 'equal',
                "value": data1[key],
            })
            continue
        else:
            list_diff.append({
                "key": key,
                "type": 'updated',
                "value1": data1[key],
                "value2": data2[key],
            })
    return list_diff



import itertools


INDENT = 4
SHIFT_LEFT = 2
DEPTH_STEP = 1


def make_stylish(value, replacer=' ', spaces_count=4):
    """Formatter function. Describes changes to each key.
    Returns a string.
    """
    def iter_(current_value, depth):

        deep_indent_size = depth * spaces_count
        left_shift = replacer * (deep_indent_size - 2)
        current_indent = ((spaces_count * depth) - spaces_count) * replacer
        lines = []

        if isinstance(current_value, list):
            for i in current_value:
                if i['type'] == 'nested':
                    lines.append(f"{left_shift}  {i['key']}: {iter_(i['value'], depth + DEPTH_STEP)}") # noqa:501
                elif i['type'] == 'added':
                    lines.append(f"{left_shift}+ {i['key']}: {to_str(i['value'], depth + DEPTH_STEP)}") # noqa:501
                elif i['type'] == 'removed':
                    lines.append(f"{left_shift}- {i['key']}: {to_str(i['value'], depth + DEPTH_STEP)}") # noqa:501
                elif i['type'] == 'equal':
                    lines.append(f"{left_shift}  {i['key']}: {to_str(i['value'], depth + DEPTH_STEP)}") # noqa:501
                elif i['type'] == 'updated':
                        lines.append(f"{left_shift}- {i['key']}: {to_str(i['value1'], depth + DEPTH_STEP)}") # noqa:501
                        lines.append(f"{left_shift}+ {i['key']}: {to_str(i['value2'], depth + DEPTH_STEP)}") # noqa:501
        result = itertools.chain("{", lines, [current_indent + "}"])
        return '\n'.join(result)

    return iter_(value, DEPTH_STEP)


def to_str(value, depth=0, replacer=' ', spaces_count=4):
    """Converts a value of different types to the required format"""
    deep_indent_size = depth * spaces_count
    deep_indent = replacer * deep_indent_size
    current_indent = ((spaces_count * depth) - spaces_count) * replacer
    if isinstance(value, dict):
        lines = []
        for k, v in value.items():
            if isinstance(v, dict):
                lines.append(f"{deep_indent}{k}: {to_str(v, depth + DEPTH_STEP)}") # noqa:501
            else:
                lines.append(f"{deep_indent}{k}: {to_str(v)}")
        result = itertools.chain("{", lines, [current_indent + "}"])
        return '\n'.join(result)
    elif value is True or value is False:
        return str(value).lower()
    elif value is None:
        return 'null'
    return str(value)



from gendiff.data import get_data
from gendiff.formatters import get_formatter
from gendiff.differ import get_different


def generate_diff(file_path1, file_path2, format_name='stylish'):
    diff = get_different(get_data(file_path1), get_data(file_path2))
    format = get_formatter(format_name)
    return format(diff)



from setuptools import setup, find_packages

setup(
    name='gendiff_dmitriev',
    version='0.1.0',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'gendiff=gendiff.scripts.gendiff:main',
        ],
    },
    install_requires=[
        'PyYAML>=6.0.1',
        'pytest>=8.3.2'        
    ],
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.10',
)



import json
import yaml


def get_data(filepath):
    """Returns data from a file."""
    if get_format_file(filepath) == 'yaml':
        file_data = yaml.safe_load(open(filepath).read())
    else:
        file_data = json.load(open(filepath))
    return file_data


def get_format_file(filepath):
    """Returns the file format."""
    if filepath.endswith('.yaml'):
        return 'yaml'
    elif filepath.endswith('.yml'):
        return 'yaml'
    elif filepath.endswith('.json'):
        return 'json'
    else:
        raise ValueError('unsupported file format')
